#include <QFileDialog>
#include <QSettings>
#include <QtNetwork>
#include <QMessageBox>
#include "mainwindow.h"
#include "ui_mainwindow.h"
//#include <math.h>
#include "hidapi.h"
#include "fe_interface.h"
#include "eeprom.h"
#include "tcp_shell.h"
#include "udp_client.h"
#include "version.h"
#include "filter_utility.h"

//#if defined (_WINRAD_PLUGIN_) || (_LINRAD_PLUGIN_)
    #include "wr_plugin.h"
//#endif
#define FREQ_TUNE_SHIFT 5
#define RF_GAIN_INIT_VALUE 103
extern void (* Callback_Winrad)(int, int, float, short *);
extern char pStreamBuffer[];

QSettings settings("./sdr_config.ini",QSettings::IniFormat);

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
    {
        ui->setupUi(this);
//        ui->tabWidget->removeTab(2);
        on_Application_Load();
        TimerID_20ms = startTimer(20);
        TimerID_100ms = startTimer(200);
        this->setWindowTitle(QString("SDR Network Control Box x2 (v.%1)").arg(_SOFTWARE_VERSION_,4));
    }

MainWindow::~MainWindow()
{
//    if(MainTimerID) killTimer(MainTimerID);
    if(fe_handle) hid_close(fe_handle);
    UDPApp.udp_stop_flag = 1;
    UDP_Close();
    UDPApp.wait(1000);
//    if (tcpSocket) delete tcpSocket;
    delete ui;
}

void MainWindow::on_Application_Load()
{
    int res=0;
    int rf_gain = RF_GAIN_INIT_VALUE;
    int fe_gain = 0;
    char stdmsg[255] = {0};
    QString *msg;
    tcpSocket = NULL;
    hw_cpld_version = 0;
    frequency = settings.value("frequency", 28050000).toInt();
    frequency_channel1 = frequency;
    frequency_channel2 = settings.value("frequency_channel2",28050000).toInt();
//    if(!frequency) frequency = 28050000;
    heterodyne_frequency = settings.value("heterodyne_frequency").toInt();
    fe_main_clock_freq = 80000011;//settings.value("fe_main_clock_freq").toUInt();
    if(settings.value("sdr_sample_rate").isNull() == true)
    {
        settings.setValue("sdr_sample_rate", DEFAULT_SAMPLE_RATE);
    }
    SDR_sample_rate = settings.value("sdr_sample_rate").toUInt();
    ui->lineEdit_network_Sample_Rate->setText(QString("%1").arg(SDR_sample_rate,7,10));
//    Calculate_sample_rate_value(SDR_sample_rate);
    if(settings.value("sdr_ip_address").isNull() == true)
    {
        settings.setValue("sdr_ip_address", QString(DEFAULT_IP_ADDRESS));
    }
    if(settings.value("sdr_ip_mask").isNull() == true)
    {
        settings.setValue("sdr_ip_mask", QString(DEFAULT_IP_MASK));
    }
    if(settings.value("gateway_ip_address").isNull() == true)
    {
        settings.setValue("gateway_ip_address",QString(DEFAULT_GATEWAY_IP));
    }
    if(settings.value("udp_port").isNull() == true)
    {
        settings.setValue("udp_port", DEFAULT_PORT_NUMBER);
    }
    if(settings.value("sdr_vco_voltage").isNull() == true)
    {
        settings.setValue("sdr_vco_voltage", 1);
    }
    VCO_voltage = settings.value("sdr_vco_voltage").toUInt();
    ui->spinBox_VCO_comp->setValue(VCO_voltage);

    Show_Main_Configuration();
    alias_inversion_flag = settings.value("alias_inversion_flag").toInt();
    if(!alias_inversion_flag) alias_inversion_flag = 1;
    inversion_flag = settings.value("inversion_flag").toInt();
    if(!inversion_flag) inversion_flag = 1;
    external_change_flag = 1;
    network_enable_flag = settings.value("network_enable_flag").toInt();
    hid_change_tcp_network_flag(network_enable_flag);
//    ui->led_network_link->setPixmap(QPixmap(QString::fromUtf8(":/new/prefix1/green.png")));
    if(network_enable_flag == 1)
    {
        broadcast_flag = settings.value("broadcast_flag",0).toUInt();
        if(broadcast_flag == 1)
        {
            ui->checkBox_broadcast_UDP->setChecked(true);
        }
        else
        {
            ui->checkBox_broadcast_UDP->setChecked(false);
        }
        broadcast_slave = settings.value("broadcast_slave").toUInt();
        if(settings.value("broadcast_slave").isNull() == true)
        {
            settings.setValue("broadcast_slave", 0);
        }
        if(broadcast_slave == 1)
        {
            ui->checkBox_broadcast_slave->setChecked(true);
        }
        else
        {
            ui->checkBox_broadcast_slave->setChecked(false);
        }
        multicast_flag = settings.value("multicast",0).toUInt();
        if(multicast_flag != 0)
        {
            ui->checkBox_multicast->setChecked(true);
//            on_checkBox_multicast_toggled(true);
        }
        else
        {
            ui->checkBox_multicast->setChecked(false);
//            on_checkBox_multicast_toggled(false);
        }
        UDPApp.udp_multicast_flag = multicast_flag;
        UDP_Port = settings.value("udp_port").toUInt();
        UDP_Init(UDP_Port);
//        global_Diversity_mode = settings.value("diversity_mode",0).toUInt();
        ui->checkBox_RX_mode->setChecked(false);
//        else ui->checkBox_RX_mode->setChecked(true);
//        SetDiversityMode(0);
        global_RX_Channel_number = settings.value("rx_channel_number",AFEDRI_CH1).toUInt();
        if(global_RX_Channel_number == AFEDRI_CH1)
        {
            ui->radioButton_RX_CH1->setChecked(true);
            //            on_radioButton_RX_CH1_toggled(true);
        }
        else ui->radioButton_RX_CH2->setChecked(true);//on_radioButton_RX_CH1_toggled(false);
//        SetRxChannelNumber(global_RX_Channel_number);
        global_Diversity_mode = settings.value("diversity_mode",DUAL_CHANNEL_MODE_OFF).toUInt();
        tcp_clear_pending_flag();
        ui->checkBox_enable_network->setCheckState(Qt::Checked);
        SDR_IP_Address.setAddress(settings.value("sdr_ip_address").toString());
        SDR_IP_Mask.setAddress(settings.value("sdr_ip_mask").toString());
//        DST_IP_Address.setAddress(settings.value("dst_ip_address").toString());
        Gateway_IP_Address.setAddress(settings.value("gateway_ip_address").toString());
        if(broadcast_slave == 0)
        {
            tcpSocket = new QTcpSocket(this);
            connect(tcpSocket, SIGNAL(readyRead()), this, SLOT(readTCPdata()));
            connect(tcpSocket, SIGNAL(error(QAbstractSocket::SocketError)),
                     this, SLOT(displayError(QAbstractSocket::SocketError)));
//        connect(tcpSocket, SIGNAL(connected()), this, SLOT(TCP_connected()));
//        connect(tcpSocket, SIGNAL(disconnected()), this, SLOT(TCP_disconnected()));
            tcpSocket->connectToHost(SDR_IP_Address, UDP_Port);
            tcpSocket->waitForConnected(1000);
            if(settings.value("bypass_lpf").isNull() == true)
            {
                settings.setValue("bypass_lpf", 0);
                bypass_lpf = 0;
            }
            else bypass_lpf = settings.value("bypass_lpf").toUInt();
            if(bypass_lpf == 1)
            {
                ui->checkBox_bypass_lpf->setChecked(true);
            }
            else
            {
                ui->checkBox_bypass_lpf->setChecked(false);
            }
        }
        else
        {
            ui->pushButton->hide();
            ui->pushButton_2->hide();
            ui->led_2->hide();
            ui->radioButton_2->hide();
        }
            if(Callback_Winrad != NULL) Callback_Winrad(EXTIO_ASYNC_MESSAGE, EXTIO_SR_CHANGED, 0 ,0);
    }
    else
    {
        ui->checkBox_enable_network->setCheckState(Qt::Unchecked);
    }
    Show_Network_Configuration();
    //displayError(QAbstractSocket::UnknownSocketError);
    if(settings.value("inversion_mode_flag").isNull() == true)
    {
        settings.setValue("inversion_mode_flag", 0);
    }
    if(settings.value("transverter_mode_flag").isNull() == true)
    {
        settings.setValue("transverter_mode_flag", 0);
    }
    rx_frequency = inversion_flag * frequency + heterodyne_frequency;
//    settings.value("transverter_mode_flag").toInt();
    if(settings.value("tcvr_heterodyne_frequency").isNull() == true)
    {
        settings.setValue("tcvr_heterodyne_frequency", DEFAULT_TCVR_HETERODYNE_FREQUENCY);
    }

    if(settings.value("transverter_mode_flag").toInt() != 0)//(ui->checkBox->isChecked()== true) || (rx_frequency > TRANSVERTER_MODE_LIMIT))
    {
        ui->checkBox->setChecked(true);
//        ui->lineEdit_2->setText(settings.value("heterodyne_frequency").toString());
        SetTransverterMode();
    }
    else
    {
        ui->lineEdit_2->hide();
        ui->label_5->hide();
        ui->checkBox_inversion_flag->hide();
    }
    rf_gain = settings.value("rf_gain").toInt();
    ui->dial_2->setValue(rf_gain);
    fe_gain = settings.value("fe_gain").toInt();
    ui->dial_FE_gain->setValue(fe_gain);
    freq_tune_step = 10;
    ui->lcdNumber1->display((int)rx_frequency);
    ui->lcdNumber3->display((int)frequency);
    if(network_enable_flag == 0)
    {
        res =  HID_Device_Init(stdmsg);
        msg = new QString(stdmsg);//::fromAscii(stdmsg);
    }
    else
    {
        res = 1;

        if(broadcast_slave == 0)
        {
            msg = new QString("AFEDRI SDR Nework");
        }
        else
        {
            msg = new QString("Broadcast Slave");
            ui->led_1->setPixmap(QPixmap(QString::fromUtf8(":/new/prefix1/green.png")));
        }
    }
    ui->radioButton->setText(*msg);
    on_horizontalSlider_tune_step_valueChanged(FREQ_TUNE_SHIFT);
//    pStreamBuffer = (short *) sharedMemory.data();
    if(res)
    {
           HID_Get_Init_Status();
//           on_dial_FE_gain_valueChanged(fe_gain);
//           on_dial_2_valueChanged(rf_gain);
    }
    delete msg;
}

void MainWindow::timerEvent(QTimerEvent *event)
{
    int res=0;
    char buf[HID_COMMAND_MAX_SIZE+16]={0};
    wchar_t *err = NULL;
    unsigned long result=0;
    int tmp_vco = 0;
    unsigned char gen_command = 0;
    static volatile int hid_changed_freq = 0;
    static  int hid_command_pending_state = 0;
    static int init_state = 0;
    static int tcp_pending_command_accu=0;
    unsigned long tmp_eeprom_data = 0;
    unsigned short tmp_eeprom_address = 0;
    static long tmp_sample_rate = 0;
    static int sr_counter = 0;
    int tmp_sr_mult = 126;
//    int fe_gain=0;
//    int rf_gain_dac=0;
    QString msg_r;
    hid_change_tcp_network_flag(network_enable_flag);
    if(event->timerId() == TimerID_100ms )
    {

        if(sr_counter < 10)
        {
            tmp_sample_rate += UDPApp.packet_counter;
            sr_counter++;
        }
        else
        {
            if(global_Diversity_mode == DUAL_CHANNEL_MODE_OFF) tmp_sr_mult = 126;
            else tmp_sr_mult = 63;
            ui->lcdNumber_packet_counter->display((int)tmp_sample_rate * tmp_sr_mult);
            tmp_sample_rate = 0;
            sr_counter = 0;
        }
        UDPApp.packet_counter = 0;
        //ui->lcdNumber_udp_error_counter->display(UDPApp.error_counter);

        if(UDPApp.udp_protocol_id == UDP_PROTCOL_ID2)
        {
            if((broadcast_slave == 1) || (broadcast_flag == 1))
            {
                if(global_Diversity_mode != DUAL_CHANNEL_MODE_OFF)
                ui->lcdNumber1_Second_NCO2->display((int)UDPApp.udp_broadcast_freq2);
            }
#if defined (_WINRAD_PLUGIN_ ) || defined (_LINRAD_PLUGIN_)
                        if(Callback_Winrad != NULL) Callback_Winrad(EXTIO_ASYNC_MESSAGE, EXTIO_FREQ_CHANGED, 0 ,0);
#endif
            if(broadcast_slave == 1)
            {
            //ui->lcdNumber1->display((int)UDPApp.udp_broadcast_freq2);
                if(global_RX_Channel_number == AFEDRI_CH2)//frequency != (int)UDPApp.udp_broadcast_freq2)
                {
                    frequency = (int)UDPApp.udp_broadcast_freq2;
                    external_change_flag = 0;
                }
                else
                {
                    frequency = (int)UDPApp.udp_broadcast_freq1;
                    external_change_flag = 0;
                }
                if(SDR_sample_rate != UDPApp.udp_broadcast_sample_rate)
                {
                    SDR_sample_rate = UDPApp.udp_broadcast_sample_rate;
                    ui->lineEdit_network_Sample_Rate->setText(QString("%1").arg(SDR_sample_rate,8,10));
                    if(Callback_Winrad != NULL) Callback_Winrad(EXTIO_ASYNC_MESSAGE, EXTIO_SR_CHANGED, 0 ,0);
                }
                if(1)//global_Diversity_mode != UDPApp.udp_broadcast_channel)
                {
                    global_Diversity_mode = UDPApp.udp_broadcast_channel;
                    switch(global_Diversity_mode)
                    {
                        case DUAL_CHANNEL_MODE_OFF:
                            ui->checkBox_RX_mode->setChecked(false);
                            on_checkBox_RX_mode_toggled(false);
                            ui->checkBox_Diversity_mode_2->setChecked(false);
                            break;
                        case DUAL_CHANNEL_MODE:
                            ui->checkBox_RX_mode->setChecked(true);
                            on_checkBox_RX_mode_toggled(true);
                            ui->checkBox_Diversity_mode_2->setChecked(false);
                            break;
                        case DIVERSITY_MODE:
                            ui->checkBox_RX_mode->setChecked(true);
                            on_checkBox_RX_mode_toggled(true);
                            ui->checkBox_Diversity_mode_2->setChecked(true);
                            break;
                    }
//                UDPApp.SetDiversityMode(global_Diversity_mode);
                }
            }
        }
    }
    if(event->timerId() == TimerID_20ms )
    {
    if((network_enable_flag == 1) && (tcpSocket != NULL))
    {
        if(tcpSocket->state() == QAbstractSocket::UnconnectedState)
        {
            ui->led_network_link->setPixmap(QPixmap(QString::fromUtf8(":/new/prefix1/red.png")));
            tcp_clear_pending_flag();
#ifndef _TCP_DISCONNECTION_
//            tcpSocket->connectToHost(SDR_IP_Address, UDP_Port);
#endif
        }
        else
        {
            ui->led_network_link->setPixmap(QPixmap(QString::fromUtf8(":/new/prefix1/green.png")));
            if(tcp_get_pending_flag() == 1)
            {
                if(tcp_pending_command_accu++ > 100)
                {
                    tcp_clear_pending_flag();
                    tcp_pending_command_accu = 0;
                }
                return;
            }
            else
            {
                TCP_write(NULL, -1);
            }
        }
    }

    if((fe_handle != 0) || (network_enable_flag == 1))
    {
             if(fe_handle) err = (wchar_t *)hid_error(fe_handle);
             if(global_device_PID == HID_DEVICE_SDR_NET_PID)
             {
                 ui->tab_5->setEnabled(true);
             }
             else
             {
                 ui->tab_5->setDisabled(true);
                 network_enable_flag = 0;
             }
    }
    else
    {
             ui->radioButton->setText("SDR is not found!");
             ui->led_1->setPixmap(QPixmap(QString::fromUtf8(":/new/prefix1/red.png")));
             ui->radioButton_2->setText("FE disconnected");
             ui->led_2->setPixmap(QPixmap(QString::fromUtf8(":/new/prefix1/red.png")));
             on_pushButton_clicked();
    }

    if(err == NULL)//!res && hid_status)
    {
            if(init_state) hid_command_pending_state = HID_Init_Global_Parameters(&init_state, hid_command_pending_state);
            res = HID_Read_Data(buf);
            if(res > 0)
            {
                    ui->led_1->setPixmap(QPixmap(QString::fromUtf8(":/new/prefix1/green.png")));
                    switch(buf[0])
                    {
                            case 6:
                                    result = HID_Get_Generic_Report(&gen_command, (const unsigned char *)buf);
                                    switch(gen_command)
                                    {
                                        case HID_GENERIC_GET_INIT_STATUS_COMMAND:
                                            if(result)
                                            {
                                               msg_r = QString("FE Init error = %1").arg(result,0,10);
                                               ui->radioButton_2->setText(msg_r);
                                               ui->led_2->setPixmap(QPixmap(QString::fromUtf8(":/new/prefix1/red.png")));
                                            }
                                            else
                                            {
                                               ui->radioButton_2->setText("FE is working properly");
                                               ui->led_2->setPixmap(QPixmap(QString::fromUtf8(":/new/prefix1/green.png")));
                                               init_state = 1;
                                               old_frequency =0;
                                           }
                                            hid_command_pending_state = 0;
                                            break;
                                        case HID_GENERIC_VER_COMMAND:
                                            msg_r = QString("FirmWare Verision: %1").arg(result,4,16, QLatin1Char('0'));
                                            hid_command_pending_state = 0;
                                            ui->label_6->setText(msg_r);
                                            break;
                                        case HID_GENERIC_GET_BROADCAST_STATE_COMMAND:
                                            hid_command_pending_state = 0;
                                            if(broadcast_flag == 1 && result == 1)
                                            {
//                                                tcpSocket->disconnectFromHost();
                                            }
                                            broadcast_flag = result;
                                            if(result) ui->checkBox_broadcast_UDP->setChecked(true);
                                            else ui->checkBox_broadcast_UDP->setChecked(false);
                                            break;
                                        case HID_GENERIC_SYNC_WORD_COMMAND:
                                            msg_r = QString("Sync Word: %1\n\r").arg(result,8,16);
                                            hid_command_pending_state = 0;
                                            ui->plainTextEdit->insertPlainText(msg_r);
                                            break;
                                        case HID_GENERIC_HW_CPLD_SN_COMMAND:
                                            hw_cpld_version = result;
                                            msg_r = QString("HW/CPLD version: %1\n\r").arg(hw_cpld_version,8,16,QLatin1Char('0'));
                                            hid_command_pending_state = 0;
                                            ui->label_hw_cpld_sn->setText(msg_r);
                                            break;
                                        case HID_GENERIC_OVERLAOD_STATE:
                                            msg_r = QString("FirmWare Version: %1").arg(result,4,16);
                                            hid_command_pending_state = 0;
                                            break;
                                        case HID_GENERIC_GET_DIVERSITY_MODE:
                                            switch(result)
                                            {
                                                case DUAL_CHANNEL_MODE_OFF:
                                                    msg_r = QString("Single Channel RX mode");
                                                    ui->checkBox_RX_mode->setChecked(false);
                                                      on_checkBox_RX_mode_toggled(false);
    //                                                ui->checkBox_Diversity_mode_2->hide();
//                                                    ui->tab_6->setEnabled(false);
                                                    break;
                                                case DIVERSITY_MODE:
                                                    msg_r = QString("Dual Channel RX mode");
                                                    ui->checkBox_RX_mode->setChecked(true);
                                                    ui->checkBox_Diversity_mode_2->setChecked(true);
  //                                                  ui->on_checkBox_RX_mode_toggled(true);
                                                    break;
                                                case DUAL_CHANNEL_MODE:
                                                    msg_r = QString("Dual Channel RX mode");
                                                    ui->checkBox_RX_mode->setChecked(true);
//                                                    ui->on_checkBox_RX_mode_toggled(true);
                                                    ui->checkBox_Diversity_mode_2->setChecked(false);
//                                                    ui->on_checkBox_Diversity_mode_2_toggled(false);
                                                    break;
                                            }
                                                global_Diversity_mode = result;
                                                ui->label_diversity_mode->setText(msg_r);
                                                hid_command_pending_state = 0;
                                            break;
                                        case HID_GENERIC_GET_SR_COMMAND:
                                            if(network_enable_flag == 1)
                                            {
                                                SDR_sample_rate = settings.value("sdr_sample_rate").toUInt();
                                                tcp_send_sample_rate(SDR_sample_rate);
                                            }
                                            else SDR_sample_rate = result & 0xFFFFFF;
                                            msg_r = QString("Sample rate: %1").arg(SDR_sample_rate,7,10);
                                            ui->lineEdit_network_Sample_Rate->setText(QString("%1").arg(SDR_sample_rate,7,10));
                                            ui->label_sr->setText(msg_r);
                                            hid_command_pending_state = 0;
#if defined (_WINRAD_PLUGIN_ )
                                            if(Callback_Winrad != NULL) Callback_Winrad(EXTIO_ASYNC_MESSAGE, EXTIO_SR_CHANGED, 0 ,0);
#endif
                                            break;
                                        case HID_GENERIC_CONSOLE_IO_COMMAND:
                                            ui->plainTextEdit->insertPlainText(QString::fromAscii((char *) &result, 1));
                                            hid_command_pending_state = 0;
                                            break;
                                        case HID_GENERIC_GET_SDR_IP_ADDRESS:
                                            SDR_IP_Address.setAddress(result);
                                            hid_command_pending_state = 0;
//                                            msg_r = QString("%1").arg((SDR_IP_Address >> 24) & 0xFF,8,10);
                                            ui->lineEdit_sdr_ip_byte_4->setText(SDR_IP_Address.toString());//msg_r);
                                            break;
                                        case HID_GENERIC_GET_SDR_IP_MASK:
                                            SDR_IP_Mask.setAddress(result);
                                            hid_command_pending_state = 0;
//                                            msg_r = QString("%1").arg((SDR_IP_Mask >> 24) & 0xFF,8,10);
                                            ui->lineEdit_sdr_ip_mask_byte_4->setText(SDR_IP_Mask.toString());//msg_r);
                                            break;
                                        case HID_GENERIC_GET_GATEWAY_IP_ADDRESS:
                                            Gateway_IP_Address.setAddress(result);
                                            hid_command_pending_state = 0;
//                                            msg_r = QString("%1").arg((Gateway_IP_Address >> 24) & 0xFF,8,10);
                                            ui->lineEdit_gateway_byte_4->setText(Gateway_IP_Address.toString());
                                            break;
                                        case HID_GENERIC_GET_DST_IP_ADDRESS:
                                            DST_IP_Address.setAddress(result);
                                            hid_command_pending_state = 0;
//                                            msg_r = QString("%1").arg((DST_IP_Address >> 24) & 0xFF,8,10);
                                            ui->lineEdit_dst_ip_byte_4->setText(DST_IP_Address.toString());
                                            break;
                                        case HID_GENERIC_GET_DST_PORT:
                                            UDP_Port = result;
                                            hid_command_pending_state = 0;
                                            msg_r = QString("%1").arg(UDP_Port,8,10);
                                            ui->lineEdit_udp_port->setText(msg_r);
                                            break;
                                         case HID_READ_EEPROM_COMMAND:
                                            tmp_eeprom_data = result & 0xFFFF;
                                            tmp_eeprom_address = (result >> 16) & 0xFFFF;
                                            switch(tmp_eeprom_address)
                                            {
                                                case VADDRESS_MAIN_CLOCK_FREQ_LOW_HALFWORD:
                                                    fe_main_clock_freq = tmp_eeprom_data;
                                                    break;
                                                case VADDRESS_MAIN_CLOCK_FREQ_HIGH_HALFWORD:
                                                    fe_main_clock_freq |= tmp_eeprom_data << 16;
                                                    if(fe_main_clock_freq == 0) fe_main_clock_freq = 80000000;
                                                    msg_r = QString("FrontEnd Main Clock Frequency:  %1(Hz)").arg(fe_main_clock_freq,8,10);
                                                    ui->label_11->setText(msg_r);
                                                    msg_r = QString("%1").arg(fe_main_clock_freq,8,10);
//                                                    ui->lineEdit_main_clock->setText(msg_r);
                                                    Calculate_sample_rate_value(SDR_sample_rate);
                                                    break;
                                            }
                                            hid_command_pending_state = 0;
                                            break;
                                         case HID_GENERIC_GET_AFEDRI_ID_COMMAND:
                                            hid_command_pending_state = 0;
                                            if(result == 0) ui->checkBox_Emulation_ID->setChecked(false);
                                            else ui->checkBox_Emulation_ID->setChecked(true);
                                            break;
                                         case HID_GENERIC_GET_DHCP_STATE_COMMAND:
                                            hid_command_pending_state = 0;
                                            if(result == 0) ui->checkBox_DHCP_Server->setChecked(false);
                                            else ui->checkBox_DHCP_Server->setChecked(true);
                                            break;
                                         case HID_GENERIC_GET_VCO_COMMAND:
                                            hid_command_pending_state = 0;
                                            tmp_vco = (int)result;
                                            VCO_voltage = (tmp_vco - 2048);
                                            ui->spinBox_VCO_comp->setValue(VCO_voltage);
                                            settings.setValue("sdr_vco_voltage", VCO_voltage);
                                            break;
                                         case HID_GENERIC_GET_GAIN_TABLE_COMMAND:
                                            hid_command_pending_state = 0;
                                            tmp_vco = (int)result;
                                            ui->spinBox_gain_table_0db->setValue(((tmp_vco >> 12) & 0x0F) *3 -10);
                                            ui->spinBox_gain_table_10db->setValue(((tmp_vco >> 8) & 0x0F)*3 - 10);
                                            ui->spinBox_gain_table_20db->setValue(((tmp_vco >> 4) & 0x0F)*3 - 10);
                                            ui->spinBox_gain_table_30db->setValue((tmp_vco & 0x0F)*3 - 10);
                                            break;
                                         case HID_GENERIC_GET_SERIAL_NUMBER_COMMAND:
//                                            result = 0xA1B2C3D4;
                                            msg_r = QString("%1").arg(result & 0xFFFFFFFF,8,16,QChar('0')).toUpper();
                                            hid_command_pending_state = 0;
                                            ui->label_Serial_Number_2->setText(msg_r);
                                            break;
                                         case HID_GENERIC_ACK:
                                            hid_command_pending_state = 0;
                                            break;
                                       }
                                    break;
                            case 7:
                                    frequency = HID_Read_Frequency((const unsigned char *)buf);
                                    hid_changed_freq = 1;
                                    hid_command_pending_state = 0;
                                    break;
                    }
            }
            else
            {
                if(old_frequency != frequency)
                {
                    settings.setValue("frequency", (int)frequency);
                    settings.setValue("heterodyne_frequency", (int)heterodyne_frequency);
                    settings.setValue("inversion_flag", (int)inversion_flag);
                    settings.setValue("alias_inversion_flag", (int)alias_inversion_flag);
                    if(network_enable_flag == 0)
                    {
                        if(!hid_changed_freq) HID_Set_Frequency(frequency);
                        else hid_changed_freq = 0;
                    }
                    else
                    {
                        if(!hid_changed_freq) tcp_send_frequency(frequency, global_RX_Channel_number);
                        else hid_changed_freq = 0;
                    }
                    switch(global_RX_Channel_number)
                    {
                        case AFEDRI_CH1:
                            frequency_channel1 = frequency;
                            break;
                        case AFEDRI_CH2:
                            frequency_channel2 = frequency;
                            if(global_Diversity_mode == DIVERSITY_MODE) frequency_channel1 = frequency;
                            break;
                    }

                    rx_frequency = inversion_flag * frequency + heterodyne_frequency;
                    ui->lcdNumber1->display((int)rx_frequency);
                    ui->lcdNumber3->display((int)frequency);
                    if(!external_change_flag)
                    {
#if defined (_WINRAD_PLUGIN_ ) || defined (_LINRAD_PLUGIN_)
                        if(Callback_Winrad != NULL) Callback_Winrad(EXTIO_ASYNC_MESSAGE, EXTIO_FREQ_CHANGED, 0 ,0);
#endif
                    }
                    external_change_flag = 0;
                    old_frequency = frequency;
                }

            }
    }
    else
    {
        if(broadcast_slave == 0)
        {
            QString *msg = new QString(QString::fromUtf16((ushort*)err));
             ui->radioButton->setText(*msg);
             hid_status = 0;
             //ui->radioButton->setChecked(false);
             ui->led_1->setPixmap(QPixmap(QString::fromUtf8(":/new/prefix1/red.png")));

             ui->radioButton_2->setText("FE disconnected");
             //ui->radioButton_2->setChecked(false);
             ui->led_2->setPixmap(QPixmap(QString::fromUtf8(":/new/prefix1/red.png")));

             delete msg;
         }
         else
         {
             QString *msg = new QString("Broadcats Slave mode");
              ui->radioButton->setText(*msg);
              hid_status = 0;
              //ui->radioButton->setChecked(false);
              ui->led_1->setPixmap(QPixmap(QString::fromUtf8(":/new/prefix1/green.png")));
              delete msg;
         }
         on_pushButton_clicked();
    }
    }
}

int MainWindow::HID_Init_Global_Parameters(int *init_state, int hid_command_pending)
{
    static int state_machine_counter = 0;
    int fe_gain=0;
    int rf_gain_dac=0;
//    int tmp_vco = 0;
    int result = hid_command_pending;
    if(!hid_command_pending)
    {
        switch(state_machine_counter)
        {
            case 0:
                HID_Check_FW_Version(0);
                result = 1;
                state_machine_counter++;
                break;
            case 1:
                if(global_device_PID == HID_DEVICE_ATEECS_PID) rf_gain_dac = 2 * ui->dial_2->value();
                else rf_gain_dac = ui->dial_2->value();
                if(network_enable_flag)
                {
                    tcp_set_rf_gain_state(rf_gain_dac);
                    result = 0;
                }
                else
                {
                    HID_ChangeFE_DAC(rf_gain_dac);
                    result = 1;
                }
                state_machine_counter++;
                break;
            case 2:
                fe_gain = ui->dial_FE_gain->value();
                HID_ChangeFE_Gain(fe_gain);
                result = 1;
                state_machine_counter++;
                break;
            case 3:
//                HID_Get_Main_Clock_Frequency();
                HID_Get_EEPROM_Data(VADDRESS_MAIN_CLOCK_FREQ_LOW_HALFWORD);
                result = 1;
                state_machine_counter++;
                break;
            case 4:
//                HID_Get_Main_Clock_Frequency();
                HID_Get_EEPROM_Data(VADDRESS_MAIN_CLOCK_FREQ_HIGH_HALFWORD);
                result = 1;
                state_machine_counter++;
                break;
            case 5:
                if(0)//network_enable_flag == 1)
                {
                    tcp_send_sample_rate(SDR_sample_rate);
                    result = 0;
                }
                else
                {
                    HID_Get_Sample_Rate();
                    result = 1;
                }
                state_machine_counter++;
                break;
            case 6:
                HID_Get_Diversity_Mode();
                result = 1;
                state_machine_counter++;
                break;
            case 7:
                HID_Get_SDR_IP_Address();
                result = 1;
                state_machine_counter++;
                break;
            case 8:
                HID_Get_SDR_IP_Mask();
                result = 1;
                state_machine_counter++;
                break;
            case 9:
                HID_Get_Gateway_IP_Address();
                result = 1;
                state_machine_counter++;
                break;
            case 10:
                HID_Get_DST_IP_Address();
                result = 1;
                state_machine_counter++;
                break;
            case 11:
                HID_Get_UDP_Port();
                result = 1;
                state_machine_counter++;
                break;
            case 12:
                HID_Get_Serial_Number();
                result = 1;
                state_machine_counter++;
                break;
            case 13:
                HID_Get_HW_CPLD_SN();
                result = 1;
                state_machine_counter++;
                break;
            case 14:
//                HID_Change_VCO_voltage(VCO_voltage);
                HID_Get_VCO_voltage();
                state_machine_counter++;
                result = 1;
                break;
            case 15:
                HID_Get_Gain_Table();
                state_machine_counter++;
                result = 1;
                break;
            case 16:
                HID_Set_Broadcast_Flag(broadcast_flag);
//                HID_Get_Broadcast_Flag();
                state_machine_counter++;
                result = 1;
                break;
            case 17:
                HID_Set_Multicast_Flag(multicast_flag);
                state_machine_counter++;
                result = 1;
                break;
            case 18:
                HID_Get_Emulation_ID();
                state_machine_counter++;
                result = 1;
                break;
            case 19:
                HID_Get_DHCP_Server_State();
                state_machine_counter++;
                result = 1;
                break;
            case 20:
                old_frequency=0;
                state_machine_counter = 0;
                *init_state = 0;
                break;
        }
    }
    return result;
}

void MainWindow::on_dial_valueChanged(int value)
{
    static int old_slider = 0;
    static double freq_shift = 0;
    int slider_p = ui->dial->sliderPosition();
    if(slider_p > old_slider)
    {
        if((old_slider < 10) && (slider_p > 240)) freq_shift = -1;
        else freq_shift = 1;
    }
    else
    {
        if((old_slider > 240) && (slider_p < 10)) freq_shift = 1;
        else
        {
           if((old_slider == slider_p) && (slider_p > 240)) freq_shift = 1;
            else freq_shift = -1;
        }
    }
    old_slider = slider_p;
    frequency += inversion_flag * freq_shift*freq_tune_step;//FREQ_TUNE_SHIFT;
    external_change_flag = 0;
}

void MainWindow::on_dial_2_valueChanged(int value)
{
    int gain;
    QString msg;
    if(global_device_PID == HID_DEVICE_ATEECS_PID)
    {
        gain = value * 2;
        RF_Gain = RF_Gain_Int_2_Double(gain);
        HID_ChangeFE_DAC(gain);
    }
    else
    {
        gain = value;
        RF_Gain = RF_Gain_Calc(gain);
        if(network_enable_flag == 1) tcp_set_rf_gain_state(gain);
        else HID_ChangeFE_DAC(gain);
    }
    msg = QString("%1 db").arg(RF_Gain,0,'f',2);
    settings.setValue("rf_gain", (int)value);
    ui->lcdNumber2->display(msg);
#if defined (_WINRAD_PLUGIN_) || (_LINRAD_PLUGIN_)
    if(!external_rf_gain_change_flag)
    {
        if(Callback_Winrad != NULL) Callback_Winrad(EXTIO_ASYNC_MESSAGE, EXTIO_RF_GAIN_CHANGED, 0 ,0);
    }
//    else
        if(Callback_Winrad != NULL) Callback_Winrad(EXTIO_ASYNC_MESSAGE, EXTIO_CHANGED_ATT, 0 ,0);

#endif
    external_rf_gain_change_flag = 0;
}

void MainWindow::Set_RF_Gain(int gain)
{
    int igain=0;
    igain = RF_Gain_IntDb_2_Int(gain);
    external_rf_gain_change_flag = 1;
    ui->dial_2->setValue(gain);
    settings.setValue("rf_gain", (int)igain);
    ui->lcdNumber_udp_error_counter->display(gain);
}

int MainWindow::Get_RF_Gain(void)
{
    return ui->dial_2->value();//  setValue(igain);
}

void MainWindow::on_pushButton_clicked()
{

    int res=0;
    char stdmsg[255] = {0};
    QString *msg;
    if(network_enable_flag == 0)
    {
        res =  HID_Device_Reconnect(stdmsg);
//        QThread::sleep(1000);
        msg = new QString(stdmsg);
    }
    else
    {
        if(tcpSocket->state() == QAbstractSocket::UnconnectedState)
        {
//            tcpSocket->connectToHost(SDR_IP_Address, UDP_Port);
            res = 1;
        }
        else res = 0;
        msg = new QString("SDR Network");
    }
    ui->radioButton->setText(*msg);
    delete msg;
    if(fe_handle || !res)
    {
           HID_Init_FE();
//           HID_Get_Frequency();
//           HID_Get_Main_Clock_Frequency();
    }
    //ui->pushButton->setStyleSheet("QPushButton  {color: red}");

}

void MainWindow::on_pushButton_2_clicked()
{
    HID_Init_FE();
//    HID_Get_Main_Clock_Frequency();
}


void MainWindow::on_dial_FE_gain_valueChanged(int value)
{
    int idx = value;
    QString msg = QString("%1").arg(GainTable[idx],0,'f',2);
    FE_Gain = GainIndex[idx];
    HID_ChangeFE_Gain(idx);
    ui->lcdNumber_FE_gain->display( msg);
    settings.setValue("fe_gain", (int)value);
//    numericUpDown1->Text = msg;
}


void MainWindow::on_lineEdit_returnPressed()
{
    rx_frequency = ui->lineEdit->text().toInt();
    calculate_frequency();
    old_frequency=0;
    external_change_flag = 0;
}

void MainWindow::FE_Set_New_RX_Frequency(long freq)
{
//    double dFreq = double(freq);
    rx_frequency = freq;
    calculate_frequency();
    external_change_flag = 1;
//    old_frequency = 0;
}

void MainWindow::RX_Set_Real_Frequency(long freq)
{
    ui->lcdNumber_rx->display((int)freq);
}

void MainWindow::on_horizontalSlider_tune_step_valueChanged(int value)
{
    int i;
    int max_val = 6- value;
    freq_tune_step = 1;
    for(i=0;i < max_val; i++) freq_tune_step *= 10;
    QString msg = QString("Tune step x%1").arg(freq_tune_step);
    ui->label_tune_step->setText(msg);
}

void MainWindow::on_lineEdit_2_returnPressed()
{
    heterodyne_frequency = ui->lineEdit_2->text().toInt();
    settings.setValue("tcvr_heterodyne_frequency", (int)heterodyne_frequency);
    refresh_frequency();
}

void MainWindow::on_checkBox_toggled(bool checked)
{
    if(checked == false)
    {
        ResetTransverterMode();
    }
    else
    {
        SetTransverterMode();
        refresh_frequency();

//        on_lineEdit_2_returnPressed();
    }
    external_change_flag = 0;
}

void MainWindow::SetTransverterMode()
{
    ui->lineEdit_2->show();
    ui->label_5->show();
    ui->checkBox_inversion_flag->show();
    if(settings.value("inversion_mode_flag").toInt() == 1)
    {
        inversion_flag = -1;
        ui->checkBox_inversion_flag->setChecked(true);
    }
    else
    {
        inversion_flag = 1;
    }
    settings.setValue("transverter_mode_flag", 1);
    heterodyne_frequency = settings.value("tcvr_heterodyne_frequency").toUInt();
    //msg =
    ui->lineEdit_2->setText(QString("%1").arg(heterodyne_frequency,10,10));//setText(QString("%1").arg((long)heterodyne_frequency,0,'l',2));//(heterodyne_frequency));// text().toInt();
}

void MainWindow::ResetTransverterMode()
{
    heterodyne_frequency=0;
    ui->lineEdit_2->hide();
    ui->label_5->hide();
    ui->checkBox_inversion_flag->hide();
    old_frequency = 0;//Change old_frequency to refresh LCD display
    inversion_flag = 1;//alias_inversion_flag;
    rx_frequency =  frequency;
    settings.setValue("transverter_mode_flag", (int)0);
}

void MainWindow::on_lineEdit_command_input_returnPressed()
{
    int i, text_size, data;
    text_size = ui->lineEdit_command_input->text().size();
    for(i=0;i<text_size;i++)
    {
        data = ui->lineEdit_command_input->text().data()[i].cell();
        HID_Send_console_char(data);
    }
    data = '\n';
    HID_Send_console_char(data);
}

void MainWindow::on_checkBox_HID_console_on_toggled(bool checked)
{
    if(checked)
    {
        HID_console_On(1);
    }
    else
    {
        HID_console_On(0);
    }
}

void MainWindow::on_pushButton_choose_filter_pressed()
{
    QString filename = QFileDialog::getOpenFileName(
        this,
        tr("Open Filter file"),
        QDir::currentPath(),
        tr("Filter files (*.txt);;All files (*.*)") );
    ui->lineEdit_filter_file->setText(filename);

}

void MainWindow::on_pushButton_main_clock_pressed()
{
    unsigned long main_freq = 0;
    main_freq = ui->lineEdit_command_input->text().toInt();  //lineEdit_main_clock->text().toInt();
    if(main_freq > 0 && main_freq < 101000000)
    {
        if(HID_Send_Main_Clock_Frequency(main_freq))
        {
            ui->plainTextEdit->appendPlainText("New Main clock was saved\n");
        }
        else
        {
            if(fe_handle)
            {
                ui->plainTextEdit->appendPlainText("Cannot save main clock parameters!\n");
            }
            else
            {
                settings.setValue("fe_main_clock_freq", (unsigned int)main_freq);
                ui->plainTextEdit->appendPlainText("USB disconnected!\nMain clock parameters was saved to config file!\n");
            }
        }
    }
    else
    {
        ui->plainTextEdit->appendPlainText("Main clock frequency out of limit!\n");
    }
}

void MainWindow::on_pushButton_save_default_pressed()
{
    if(HID_Send_defaults_to_EEPROM())
    {
        ui->plainTextEdit->appendPlainText("Default parameters was saved\n");
    }
    else
    {
        ui->plainTextEdit->appendPlainText("Cannot save default parameters!\n");
    }
}

void MainWindow::on_pushButton_clicked(bool checked)
{

}

void MainWindow::on_checkBox_inversion_flag_toggled(bool checked)
{
    if(checked == true)
    {
        inversion_flag= -1;
        settings.setValue("inversion_mode_flag", 1);

    }
    else
    {
        inversion_flag = 1;
        settings.setValue("inversion_mode_flag", 0);
    }
//    frequency = inversion_flag;
    refresh_frequency();
}

void MainWindow::refresh_frequency()
{
    rx_frequency = heterodyne_frequency + inversion_flag * frequency;
    old_frequency = 0;//Change old_frequency to refresh LCD display
}

void MainWindow::calculate_frequency()
{
    long tmp_val = fe_main_clock_freq/2;
    long tmp_div = rx_frequency / tmp_val;
    if(rx_frequency > tmp_val)
    {
        if((ui->checkBox->isChecked()== true) || (rx_frequency > TRANSVERTER_MODE_LIMIT))//rx_frequency > TRANSVERTER_MODE_LIMIT)
        {
            ui->checkBox->setChecked(true);
//            on_checkBox_toggled(true);
            SetTransverterMode();
            heterodyne_frequency = ui->lineEdit_2->text().toInt();
        }
        else
        {
            if(ui->checkBox->isChecked()==true)
            {
                ui->checkBox->setChecked(false);
                ResetTransverterMode();
            }
            if(tmp_div & 1)
            {
                alias_inversion_flag = -1;
                tmp_div++;
            }
            else
            {
                alias_inversion_flag = 1;
            }
            inversion_flag = alias_inversion_flag;
            heterodyne_frequency = (tmp_div >> 1) * fe_main_clock_freq;
        }
    }
    else
    {
        heterodyne_frequency = 0;
        alias_inversion_flag = 1;
        inversion_flag = 1;
    }
    frequency = labs(rx_frequency - heterodyne_frequency);
}

void MainWindow::on_pushButton_SR_pressed()
{
    unsigned long tmp_sample_rate = 0;
    int res=0;
    tmp_sample_rate = ui->lineEdit_command_input->text().toInt();
    if((tmp_sample_rate > 16000 ) && (tmp_sample_rate < 1300000))
    {
        if(global_Diversity_mode != DUAL_CHANNEL_MODE_OFF)
            res = HID_Send_Diversity_Sample_Rate(tmp_sample_rate);
        else res = HID_Send_Sample_Rate(tmp_sample_rate);

        if(res)
        {
            ui->plainTextEdit->appendPlainText("New Sample Rate was saved\n");
        }
        else
        {
            if(fe_handle)
            {
                ui->plainTextEdit->appendPlainText("Cannot save Sample Rate!\n");
            }
            else
            {
                ui->plainTextEdit->appendPlainText("USB disconnected!\nSample Rate was saved to config file!\n");
                settings.setValue("sdr_sample_rate", (unsigned int)tmp_sample_rate);
            }
        }
    }
    else
    {
        ui->plainTextEdit->appendPlainText("Sample Rate is out of range!!!\n");
    }
}


void MainWindow::on_pushButton_RX_mode_clicked()
{
    if(HID_Send_Diversity_Mode(global_Diversity_mode))
        ui->plainTextEdit->appendPlainText("New RX mode was saved\n");
    else
        ui->plainTextEdit->appendPlainText("Cannot save new RX mode!\n");

}

void MainWindow::on_pushButton_5_clicked()
{
unsigned long tmp_long=0;
QString msg_p;
    tmp_long = ui->lineEdit_udp_port->text().toInt();
    if(fe_handle)
    {
        if(UDP_Port != tmp_long)
        {
            HID_Save_UDP_Port(tmp_long);
        }
    }
    settings.setValue("udp_port", (unsigned int)tmp_long);
    msg_p = QString("UDP port: %1 was sent!").arg(tmp_long,8,10);
    ui->plainTextEdit_Network->appendPlainText(msg_p);
    SDR_IP_Address.setAddress(ui->lineEdit_sdr_ip_byte_4->text());
    tmp_long = SDR_IP_Address.toIPv4Address();
    if(fe_handle) HID_Save_SDR_IP_Address(tmp_long);
    settings.setValue("sdr_ip_address", SDR_IP_Address.toString());//(unsigned int)tmp_long);
    msg_p = QString("SDR IP: %1 was sent!").arg(tmp_long,8,16);
    ui->plainTextEdit_Network->appendPlainText(msg_p);

    SDR_IP_Mask.setAddress(ui->lineEdit_sdr_ip_mask_byte_4->text());
    tmp_long = SDR_IP_Mask.toIPv4Address();
    if(fe_handle) HID_Save_SDR_IP_Mask(tmp_long);
    settings.setValue("sdr_ip_mask", SDR_IP_Mask.toString());//(unsigned int)tmp_long);
    msg_p = QString("SDR IP mask: %1 was sent!").arg(tmp_long,8,16);
    ui->plainTextEdit_Network->appendPlainText(msg_p);

    Gateway_IP_Address.setAddress(ui->lineEdit_gateway_byte_4->text());
    tmp_long = Gateway_IP_Address.toIPv4Address();
    if(fe_handle) HID_Save_Gateway_IP_Address(tmp_long);
    settings.setValue("gateway_ip_address", Gateway_IP_Address.toString());//(unsigned int)tmp_long);
    msg_p = QString("Gateway IP: %1 was sent!").arg(tmp_long,8,16);
    ui->plainTextEdit_Network->appendPlainText(msg_p);

//    DST_IP_Address.setAddress(ui->lineEdit_dst_ip_byte_4->text());
//    tmp_long = DST_IP_Address.toIPv4Address();
//    if(fe_handle) HID_Save_DST_IP_Address(tmp_long);
//    else settings.setValue("dst_ip_address", DST_IP_Address.toString());//(unsigned int)tmp_long);
//    msg_p = QString("DST IP Address %1 was sent!").arg(tmp_long,8,16);
//    ui->plainTextEdit_Network->appendPlainText(msg_p);
}

void MainWindow::Show_Network_Configuration()
{
    QString msg_r;
    ui->lineEdit_sdr_ip_byte_4->setText(SDR_IP_Address.toString());//msg_r);
    ui->lineEdit_sdr_ip_mask_byte_4->setText(SDR_IP_Mask.toString());//msg_r);
    ui->lineEdit_gateway_byte_4->setText(Gateway_IP_Address.toString());
    ui->lineEdit_dst_ip_byte_4->setText(DST_IP_Address.toString());
    msg_r = QString("%1").arg(UDP_Port,8,10);
    ui->lineEdit_udp_port->setText(msg_r);
}

void MainWindow::on_checkBox_3_toggled(bool checked)
{
    if(checked == true)
    {
        ui->lineEdit_sdr_ip_byte_4->setReadOnly(false);
        ui->lineEdit_gateway_byte_4->setReadOnly(false);
        ui->lineEdit_sdr_ip_mask_byte_4->setReadOnly(false);
        ui->lineEdit_udp_port->setReadOnly(false);
    }
    else
    {
        ui->lineEdit_sdr_ip_byte_4->setReadOnly(true);
        ui->lineEdit_gateway_byte_4->setReadOnly(true);
        ui->lineEdit_sdr_ip_mask_byte_4->setReadOnly(true);
        ui->lineEdit_udp_port->setReadOnly(true);
    }
}

void MainWindow::on_checkBox_start_UDP_toggled(bool checked)
{
    Start_UDP_Stream(checked);
}

void MainWindow::Start_UDP_Stream(bool on_off_state)
{
    if(on_off_state == true)
    {
        if(ui->checkBox_start_UDP->isChecked() == false)
        {
           ui->checkBox_start_UDP->setCheckState(Qt::Checked);
        }
        if(network_enable_flag == 0)  HID_Start_UDP_Stream(1);
        else tcp_set_udp_rx_state(RCV_START);
    }
    else
    {
        if(ui->checkBox_start_UDP->isChecked() == true)
        {
           ui->checkBox_start_UDP->setCheckState(Qt::Unchecked);
        }
        if(network_enable_flag == 0) HID_Start_UDP_Stream(0);
        else tcp_set_udp_rx_state(RCV_STOP);
    }
}

void MainWindow::Show_Network_Message(QString message)
{
    ui->plainTextEdit_Network->appendPlainText(message);
}

void MainWindow::Show_Main_Configuration()
{
    QString msg_r;
    msg_r = QString("FrontEnd Main Clock Frequency:  %1(Hz)").arg(fe_main_clock_freq,8,10);
    ui->label_11->setText(msg_r);
    msg_r = QString("Sample rate: %1").arg(SDR_sample_rate,7,10);
    ui->label_sr->setText(msg_r);
}

void MainWindow::on_checkBox_enable_network_toggled(bool checked)
{
    static int first_start = 0;
    int tmp_state=0;
    if(checked == true)
    {
        tmp_state = 1;
        if(first_start) ui->plainTextEdit_Network->insertPlainText("Network interface was enabled!\n Pleas reload software to run with new setting!!!\n");
    }
    else
    {
        tmp_state = 0;
        if(first_start) ui->plainTextEdit_Network->insertPlainText("Network interface was disabled!\n Pleas reload software to run with new setting!!!\n");
    }
    settings.setValue("network_enable_flag", tmp_state);//network_enable_flag);
    first_start = 1;
}

void MainWindow::readTCPdata()
{
    QByteArray rx_block;
    unsigned char tmp = 0;
    int tmp_size=0;
    rx_block.resize(tcpSocket->bytesAvailable());
    rx_block = tcpSocket->readAll();
    tmp_size = rx_block.size();
    tmp = rx_block[1];
    if(( tmp >> 5) == TCP_HID_PACKET)
    {
        hid_save_recieved_tcp_packet(rx_block.data(), tmp_size);
#ifndef _TCP_DEBUG_CONSOLE_
    }
#else
        ui->plainTextEdit_Network->insertPlainText(QString("\nHID->%1 ").arg(tmp_size,2,10));
    }
    ui->plainTextEdit_Network->insertPlainText(QString("\nhex: %1 ").arg((unsigned short)rx_block[0],2,16));
    for(int i = 1; i <  tmp_size; i++)
    ui->plainTextEdit_Network->insertPlainText(QString("%1 ").arg((unsigned short)rx_block[i],2,16));
#endif
    tcp_clear_pending_flag();
#ifdef _TCP_DISCONNECTION_
    tcpSocket->disconnectFromHost();
#endif
}

void MainWindow::displayError(QAbstractSocket::SocketError error)
{
    ui->plainTextEdit_Network->insertPlainText(QString("TCP Error:  %1\n").arg(error,8,10));
}

//End of File


void MainWindow::on_lineEdit_network_Sample_Rate_returnPressed()
{
    SDR_sample_rate = ui->lineEdit_network_Sample_Rate->text().toInt();
    if(network_enable_flag == 1)
    {
        tcp_send_sample_rate(SDR_sample_rate);
    }
    else
    {
        HID_Set_New_Sample_Rate(SDR_sample_rate);
    }
    settings.setValue("sdr_sample_rate", (int)SDR_sample_rate);
    if(Callback_Winrad != NULL) Callback_Winrad(EXTIO_ASYNC_MESSAGE, EXTIO_SR_CHANGED, 0 ,0);
    Calculate_sample_rate_value(SDR_sample_rate);
}

int MainWindow::TCP_write_data(char *block, int size)
{
    return 0;
}

void MainWindow::on_spinBox_VCO_comp_valueChanged(int value)
{
    HID_Change_VCO_voltage(value);
    settings.setValue("sdr_vco_voltage", value);
}

void MainWindow::on_pushButton_VCO_save_pressed()
{
    int tmp_value = ui->spinBox_VCO_comp->value();
    HID_Save_Default_VCO_voltage(tmp_value);
    ui->plainTextEdit->appendPlainText("Default VCO voltage was saved\n");
}

void MainWindow::on_pushButton_gain_table_pressed()
{
    unsigned short gain_table=0;
    int gain_0db, gain_10db, gain_20db, gain_30db;
    gain_0db = (ui->spinBox_gain_table_0db->value() + 10)/3;
    gain_10db = (ui->spinBox_gain_table_10db->value()+10)/3;
    gain_20db = (ui->spinBox_gain_table_20db->value()+10)/3;
    gain_30db = (ui->spinBox_gain_table_30db->value()+10)/3;
    gain_table = ((gain_0db << 12) & 0xF000) | ((gain_10db << 8) & 0x0F00) | ((gain_20db << 4) & 0x00F0) | (gain_30db & 0x000F);
    HID_Save_Gain_Table(gain_table);
}

void MainWindow::Change_BPF(unsigned short band_idx)
{
    unsigned short tmp_cap = 0;
    unsigned short tmp_hcap = 0;
    current_band = band_idx & 0x00FF;
    tmp_cap = ui->spinBox_Filter_Cap->value();
    tmp_hcap = ((tmp_cap & 0x80) >> 7) | ((tmp_cap & 0x40) >> 5) | ((tmp_cap & 0x20) >> 3) | ((tmp_cap & 0x10) >> 1)
               | ((tmp_cap & 0x01) << 7) | ((tmp_cap & 0x02) << 5) | ((tmp_cap & 0x04) << 3) | ((tmp_cap & 0x08) << 1);
    tmp_cap = (tmp_hcap << 8) | current_band;
    HID_Change_BPF_Tune(tmp_cap);
}

void MainWindow::on_radioButton_Bnad_A_clicked(bool checked)
{
    if(checked == true) Change_BPF(BAND_A);
}

void MainWindow::on_radioButton_Bnad_B_clicked(bool checked)
{
    if(checked == true) Change_BPF(BAND_B);
}

void MainWindow::on_radioButton_Bnad_C_clicked(bool checked)
{
    if(checked == true) Change_BPF(BAND_C);
}

void MainWindow::on_radioButton_Bnad_D_clicked(bool checked)
{
    if(checked == true) Change_BPF(BAND_D);
}

void MainWindow::on_radioButton_Bnad_E_clicked(bool checked)
{
    if(checked == true) Change_BPF(BAND_E);
}

void MainWindow::on_spinBox_Filter_Cap_valueChanged(int value)
{
    Change_BPF(current_band);
}

void MainWindow::on_pushButton_get_sync_state_pressed()
{
    HID_Get_Sync_State();
}

void MainWindow::on_pushButton_calibration_pressed()
{
    double test_frequency =0;
    double measure_frequency =0;
    double calibrated_frequency = 0;
    int int_calibrated_frequency = 0;
    double current_main_frequency = fe_main_clock_freq;
    test_frequency = ui->lineEdit_test_freq->text().toFloat();
    measure_frequency = ui->lineEdit_measure_freq->text().toFloat();
    calibrated_frequency = current_main_frequency * ((2*test_frequency - measure_frequency) / test_frequency);
    int_calibrated_frequency = calibrated_frequency;
    ui->lineEdit_command_input->setText(QString("%1").arg(int_calibrated_frequency,9,10));
    ui->plainTextEdit->insertPlainText("Please check now \"Command Input\" text box, it contains calculated real value of SDR's Main Clock Frequecy \n\r");
    ui->plainTextEdit->insertPlainText("If you want to save calibrated Main Clock Frequency parameter in SDR's internal EEPROM, press the button \"Save Main Clock\" \n\r");
    ui->plainTextEdit->insertPlainText("It is strongly recommended also to correct Sample Rate value to match the new Main Clock frequency\n\r");
}

void MainWindow::Calculate_sample_rate_value(unsigned long sr)
{
    double dSR = sr;
    double tmp_div = 0;
    double floor_div = 0;
    double floor_SR = 0;
    unsigned long l_SR = 0;
    tmp_div = fe_main_clock_freq / (4 * dSR);
    floor_div = floor(tmp_div);
    if((tmp_div - floor_div) >= 0.5) floor_div += 1;
    dSR = fe_main_clock_freq / (4 * floor_div);
    floor_SR = floor(dSR);
    if(dSR - floor_SR >= 0.5) floor_SR += 1;
    l_SR = floor_SR;
    if(l_SR == sr)
    {
        ui->label_sample_rate_value->setText(QString("<span style=\" font-size:8pt; font-weight:600; color:green;\">Ok!</span>"));//.arg(l_SR, 9, 10));
    }
    else
    {
        ui->label_sample_rate_value->setText(QString("<span style=\" font-size:8pt; font-weight:600; color:red;\">%1</span>").arg(l_SR, 9, 10));
    }
}

void MainWindow::on_pushButton_firmware_upgrade_pressed()
{
    HID_Firmware_Upgrade();
}


void MainWindow::on_checkBox_broadcast_UDP_toggled(bool checked)
{
    int tmp_state=0;
    if(checked == true)
    {
        tmp_state = 1;
    }
    else
    {
        tmp_state = 0;
    }
    broadcast_flag = tmp_state;
    UDPApp.SetBroadCastFlag(broadcast_flag);
    HID_Set_Broadcast_Flag(tmp_state);
    settings.setValue("broadcast_flag", tmp_state);
}

void MainWindow::on_checkBox_broadcast_slave_toggled(bool checked)
{
    int tmp_state=0;
    if(checked == true)
    {
        tmp_state = 1;
    }
    else
    {
        tmp_state = 0;
    }
    broadcast_slave = tmp_state;
    settings.setValue("broadcast_slave", broadcast_slave);
}

void MainWindow::on_checkBox_bypass_lpf_toggled(bool checked)
{
    int tmp_state=0;
    if(checked == true)
    {
        tmp_state = 1;
    }
    else
    {
        tmp_state = 0;
    }
    bypass_lpf = tmp_state;
    HID_Send_LPF_Bypass(tmp_state);
    settings.setValue("bypass_lpf", bypass_lpf);
}
void MainWindow::on_checkBox_RX_mode_toggled(bool checked)
{
    int tmp=0;
    if(checked == true)
    {
        ui->checkBox_RX_mode->setText("Dual Channel RX Mode");
        if(ui->checkBox_Diversity_mode_2->isChecked() == true)
        {
            tmp = DIVERSITY_MODE;
        }
        else
        {
            tmp = DUAL_CHANNEL_MODE;
        }
        settings.setValue("diversity_mode", (unsigned int)global_Diversity_mode);
        ui->checkBox_Diversity_mode_2->show();
        ui->tab_6->setEnabled(true);
    }
    else
    {
        ui->checkBox_RX_mode->setText("Single Channel RX Mode");
        tmp = DUAL_CHANNEL_MODE_OFF;
        settings.setValue("diversity_mode", (unsigned int)global_Diversity_mode);
        ui->checkBox_Diversity_mode_2->hide();
        global_RX_Channel_number = AFEDRI_SINGLE_CHANNEL;
        ui->tab_6->setEnabled(false);
    }
    if(broadcast_slave == 0)
    {
        global_Diversity_mode = tmp;
        HID_Set_Multichannel_Mode(tmp);
    }
    UDPApp.SetDiversityMode(global_Diversity_mode);
}

void MainWindow::on_radioButton_RX_CH1_toggled(bool checked)
{
    if(checked == true)
    {
        global_RX_Channel_number = AFEDRI_CH1;
        frequency = frequency_channel1;
    }
    else
    {
        global_RX_Channel_number = AFEDRI_CH2;
        if(global_Diversity_mode == DUAL_CHANNEL_MODE)
        {
             frequency = frequency_channel2;
        }
    }
    UDPApp.SetRxChannelNumber(global_RX_Channel_number);
    settings.setValue("rx_channel_number", global_RX_Channel_number);
}


void MainWindow::on_lineEdit_NCO2_Frequency_returnPressed()
{
    int tmp=0;
    if(global_Diversity_mode == DUAL_CHANNEL_MODE)
    {
        switch(global_RX_Channel_number)
        {
            case AFEDRI_CH1:
                tmp = AFEDRI_CH2;
                break;
            case AFEDRI_CH2:
                tmp = AFEDRI_CH1;
        }
        frequency_channel2 = ui->lineEdit_NCO2_Frequency->text().toInt();
        tcp_send_frequency(frequency_channel2, tmp);
        settings.setValue("frequency_channel2", (int)frequency_channel2);
    }
}

void MainWindow::on_checkBox_Diversity_mode_2_toggled(bool checked)
{
    int tmp=0;
    if(ui->checkBox_RX_mode->isChecked() == true)//global_Diversity_mode != DUAL_CHANNEL_MODE_OFF)
    {
            if(checked == true)
            {
                tmp = DIVERSITY_MODE;
                settings.setValue("diversity_mode", (unsigned int)global_Diversity_mode);
            }
            else
            {
                tmp = DUAL_CHANNEL_MODE;
                settings.setValue("diversity_mode", (unsigned int)global_Diversity_mode);
            }
     }
     if(broadcast_slave == 0)
     {
         global_Diversity_mode = tmp;
         HID_Set_Multichannel_Mode(global_Diversity_mode);
     }
     UDPApp.SetDiversityMode(global_Diversity_mode);
}

void MainWindow::on_checkBox_multicast_toggled(bool checked)
{
    if(checked == true)
    {
        multicast_flag = 1;
    }
    else
    {
        multicast_flag = 0;
    }
    UDPApp.udp_multicast_flag = multicast_flag;
    settings.setValue("multicast", (unsigned int)multicast_flag);
    HID_Set_Multicast_Flag(multicast_flag);
}

void MainWindow::on_pushButton_ResetSDR_pressed()
{
    HID_SoftReset_SDR();
}

void MainWindow::on_checkBox_Emulation_ID_clicked(bool checked)
{
    int tmp_id;
    if(checked == true) tmp_id = 1;
    else tmp_id = 0;
//    current_id = tmp_id;
    HID_Set_ID_String(tmp_id);
}

void MainWindow::on_pushButton_SaveID_pressed()
{
    int tmp_id;
    if(ui->checkBox_Emulation_ID->isChecked() == true) tmp_id = 1;
    else tmp_id = 0;
//    current_id = tmp_id;
    HID_Save_Default_ID(tmp_id);
}

void MainWindow::on_checkBox_DHCP_Server_clicked(bool checked)
{
    int tmp_id;
    if(checked == true) tmp_id = 1;
    else tmp_id = 0;
//    current_id = tmp_id;
    HID_Set_DHCP_Server_State(tmp_id);
}
