#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QHostAddress>
#include <QtNetwork>

namespace Ui {
    class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();
    void FE_Set_New_RX_Frequency(long freq);
    void RX_Set_Real_Frequency(long freq);
    void Set_RF_Gain(int gain);
    int Get_RF_Gain(void);
    void Show_Network_Message(QString message);
    void Show_Network_Configuration();
    void Start_UDP_Stream(bool on_off_state);
    void Show_Main_Configuration();
    int TCP_write_data(char *block, int size);
    int HID_Init_Global_Parameters(int *init_state, int hid_command_pending);
    double RF_Gain;
    volatile long frequency;
    volatile long frequency_channel1;
    volatile long frequency_channel2;
    volatile long rx_frequency;
    long inversion_flag;
    long alias_inversion_flag;
    long real_rx_frequency;
    long heterodyne_frequency;
    volatile long old_frequency;
    long freq_tune_step;
    volatile int external_change_flag;
    int external_rf_gain_change_flag;
    int TimerID_20ms;
    int TimerID_100ms;
    int VCO_voltage;
    unsigned short current_band;
    unsigned long fe_main_clock_freq ;
    unsigned long hw_cpld_version;
    unsigned long SDR_sample_rate;
    short global_Diversity_mode;
    int global_RX_Channel_number;
    volatile int network_enable_flag;
    volatile int broadcast_flag;
    volatile int multicast_flag;
    volatile int broadcast_slave;
    volatile int bypass_lpf;
//    volatile int current_id;
    QHostAddress SDR_IP_Address;
    QHostAddress SDR_IP_Mask;
    QHostAddress DST_IP_Address;
    QHostAddress Gateway_IP_Address;
    QTcpSocket *tcpSocket;
    unsigned short UDP_Port;
    int TCP_command_pending;
private:
    Ui::MainWindow *ui;
protected:
    void timerEvent(QTimerEvent *event);
    void on_Application_Load();
    void refresh_frequency();
    void calculate_frequency();
    void SetTransverterMode();
    void ResetTransverterMode();
    void Calculate_sample_rate_value(unsigned long sr);
private slots:
//    void on_pushButton_5_pressed();
//    void on_lineEdit_3_returnPressed();
    void on_checkBox_DHCP_Server_clicked(bool checked);
    void on_pushButton_SaveID_pressed();
    void on_checkBox_Emulation_ID_clicked(bool checked);
    void on_pushButton_ResetSDR_pressed();
    void on_checkBox_multicast_toggled(bool checked);
    void on_checkBox_Diversity_mode_2_toggled(bool checked);
    void on_lineEdit_NCO2_Frequency_returnPressed();
    void on_radioButton_RX_CH1_toggled(bool checked);
    void on_checkBox_bypass_lpf_toggled(bool checked);
    void on_checkBox_broadcast_slave_toggled(bool checked);
    void on_checkBox_broadcast_UDP_toggled(bool checked);
    void on_pushButton_firmware_upgrade_pressed();
    void on_pushButton_calibration_pressed();
    void on_pushButton_get_sync_state_pressed();
    void Change_BPF(unsigned short band_idx);
    void on_spinBox_Filter_Cap_valueChanged(int value);
    void on_radioButton_Bnad_E_clicked(bool checked);
    void on_radioButton_Bnad_D_clicked(bool checked);
    void on_radioButton_Bnad_C_clicked(bool checked);
    void on_radioButton_Bnad_B_clicked(bool checked);
    void on_radioButton_Bnad_A_clicked(bool checked);
    void on_pushButton_gain_table_pressed();
    void on_pushButton_VCO_save_pressed();
    void on_spinBox_VCO_comp_valueChanged(int value);
    void on_lineEdit_network_Sample_Rate_returnPressed();
    void on_checkBox_enable_network_toggled(bool checked);
    void on_checkBox_start_UDP_toggled(bool checked);
    void on_checkBox_3_toggled(bool checked);
    void on_pushButton_5_clicked();
    void on_pushButton_RX_mode_clicked();
    void on_checkBox_RX_mode_toggled(bool checked);
    void on_pushButton_SR_pressed();
    void on_checkBox_inversion_flag_toggled(bool checked);
    void on_pushButton_clicked(bool checked);
    void on_pushButton_save_default_pressed();
    void on_pushButton_main_clock_pressed();
    void on_pushButton_choose_filter_pressed();
    void on_checkBox_HID_console_on_toggled(bool checked);
    void on_lineEdit_command_input_returnPressed();
    void on_checkBox_toggled(bool checked);
    void on_lineEdit_2_returnPressed();
    void on_horizontalSlider_tune_step_valueChanged(int value);
    void on_lineEdit_returnPressed();
    void on_dial_FE_gain_valueChanged(int value);
    void on_pushButton_2_clicked();
    void on_pushButton_clicked();
    void on_dial_2_valueChanged(int value);
    void on_dial_valueChanged(int value);
    void readTCPdata();
    void displayError(QAbstractSocket::SocketError error);
  };

#endif // MAINWINDOW_H
